package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.BeanMapperUtils;
import com.bizmatics.mhfire.persistence.mapper.BsSupervisionAllMapper;
import com.bizmatics.mhfire.persistence.mapper.FireStatisticsMapper;
import com.bizmatics.mhfire.persistence.mapper.UnitMapper;
import com.bizmatics.mhfire.persistence.mapper.po.BsSupervisionAllPO;
import com.bizmatics.mhfire.persistence.mapper.po.UnitPO;
import com.bizmatics.mhfire.service.BsSupervisionAllService;
import com.bizmatics.mhfire.service.UnitService;

import com.bizmatics.mhfire.service.enums.BsSaOneCode;
import com.bizmatics.mhfire.service.enums.BsSaTwoCode;
import com.bizmatics.mhfire.service.vo.UnitAlCheckVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author yq
 * @date 2021/5/27 13:37
 */
@Service
public class UnitServiceImpl implements UnitService {

    @Autowired
    private UnitMapper unitMapper;
    @Autowired
    private BsSupervisionAllMapper bsSupervisionAllMapper;

    @Autowired
    private FireStatisticsMapper fireStatisticsMapper;

    @Autowired
    private BsSupervisionAllService bsSupervisionAllService;
    @Override
    public UnitPO getOne(UnitPO unitPo) {
        Map<String, Object> map = unitMapper.selectOne(unitPo);
        return enhanceUnitPo(map);
    }

    @Override
    public CommonPage<UnitPO> page(Integer current, Integer size) {
        Page<Map<String, Object>> page = new Page<>(current, size);
        page = unitMapper.page(page);
        List<UnitPO> unitPoList = new ArrayList<>();
        page.getRecords().forEach(stringObjectMap -> unitPoList.add(enhanceUnitPo(stringObjectMap)));
        return new CommonPage<>(unitPoList, page.getTotal(), page.getSize(), page.getCurrent());

    }

    @Override
    public UnitAlCheckVO getUnitAlCheck(String unitId, Date startTime, Date endTime) {
        UnitPO unitPo = new UnitPO();
        unitPo.setId(unitId);
        UnitAlCheckVO unitAlCheckVo = BeanMapperUtils.map(getOne(unitPo), UnitAlCheckVO.class);
        Integer checkNumber = bsSupervisionAllMapper.selectCount(null, null, startTime, endTime, unitId);
        unitAlCheckVo.setCheckNumber(checkNumber);
        //历史火灾统计
        Integer fireCount = fireStatisticsMapper.selectCount(startTime, endTime, null, null, unitId);
        unitAlCheckVo.setHistoryFire(fireCount);
        //审核时间
        BsSupervisionAllPO byRecentlyOne = bsSupervisionAllService.getByRecentlyOne(BsSaOneCode.A_P.getLetter(), BsSaTwoCode.J_G_S_G_AUDIT.getLetter());
        unitAlCheckVo.setAuditTime(Optional.ofNullable(byRecentlyOne).map(BsSupervisionAllPO::getProductCreatTime).orElse(null));
        //验收
        BsSupervisionAllPO acceptOne = bsSupervisionAllService.getByRecentlyOne(BsSaOneCode.A_P.getLetter(), BsSaTwoCode.J_G_S_G_ACCPET.getLetter());
        unitAlCheckVo.setAcceptTime(Optional.ofNullable(acceptOne).map(BsSupervisionAllPO::getProductCreatTime).orElse(null));
        //行政许可安检时间
        BsSupervisionAllPO serviceByRecentlyOne = bsSupervisionAllService.getByRecentlyOne(BsSaOneCode.A_P.getLetter(), BsSaTwoCode.R_C_C.getLetter());
        unitAlCheckVo.setSecurityTime(Optional.ofNullable(serviceByRecentlyOne).map(BsSupervisionAllPO::getProductCreatTime).orElse(null));
        //TODO 历史检查情况,历史隐患次数找不见对应记录(数据在行政处罚里面)
        return unitAlCheckVo;
    }

    /**
     * map转换为unitpo
     * @param unitPoMap
     * @return
     */
    public UnitPO enhanceUnitPo(Map<String,Object> unitPoMap){
        UnitPO unitPo = new UnitPO();
        Optional.ofNullable(unitPoMap).ifPresent(
                unit -> {
                    unitPo.setId(unit.get("ID").toString());
                    unitPo.setName(Optional.ofNullable(unit.get("单位名称")).orElse("").toString());
                    unitPo.setAddress(unit.get("单位地址").toString());
                    unitPo.setPrincipal(unit.get("法人代表或主要负责人").toString());
                    unitPo.setPhone(unit.get("单位电话").toString());
                }
                );
        return unitPo;
    }

}
